import os
import zlib

from keyczar.keys import AesKey


def encrypt_config(config, keyfile=None):
    """
    Encrypt a config file.

    Args:
        config (str): The path to the config file to be encrypted.
    Kwargs:
        keyfile (str): A previously generated encryption keyfile.
    Returns:
        string or None. The path to the encrypted file, or None.
    Raises:
       ConfigError
    """
    if config is None:
        raise ConfigError("You must supply the path to your config file.")
    elif not os.path.exists(config):
        raise ConfigError("Application config doesn't exist at {0}".format(
            config
        ))

    if keyfile is None or os.path.exists(keyfile) is False:
        key = None

        if os.path.exists("key.pem"):
            try:
                with open("key.pem", "r") as f:
                    data = f.read()
                key = AesKey.Read(data)

                res = raw_input(("Encryption key appears to exist in key.pem. "
                                 "Use this? [Y/n] "))
                if len(res) > 0 and not res.lower().startswith('y'):
                    key = None
            except (ValueError, KeyError):
                pass

        if key is None:
            res = raw_input("Encryption key not provided. Create? [Y/n] ")
            if len(res) > 0 and not res.lower().startswith('y'):
                return None

            key = AesKey.Generate()
            with open("key.pem", "w") as f:
                f.write(str(key))
    else:
        with open(keyfile, "r") as f:
            data = f.read()

        key = AesKey.Read(data)

    with open(config, "r") as f:
        data = f.read()
    with open("{0}.encrypted".format(config), "w") as f:
        f.write(key.Encrypt(zlib.compress(data)))

    return config


def decrypt_config(config, keyfile):
    """
    Decrypt a config file.

    Args:
        config (str): The path to the config file to be encrypted.
        keyfile (str): The keyfile to use for decryption.
    Kwargs:
    Returns:
        string or None. The path to the decrypted file, or None.
    Raises:
       ConfigError
    """
    if config is None:
        raise ConfigError(("You must supply the path to your encrypted config "
                           "file."))
    else:
        if config.endswith(".encrypted"):
            config = config[0:-10]

        if not os.path.exists("{0}.encrypted".format(config)):
            raise ConfigError("Encrypted config doesn't exist at {0}.encrypted"
                              .format(config))

    if keyfile is None:
        raise ConfigError("Encryption keys are required for decryption!")
    elif not os.path.exists(keyfile):
        raise ConfigError("Encryption key doesn't exist at {0}".format(
            keyfile
        ))

    if os.path.exists(config):
        res = raw_input("Application config already exists. Overwrite? [y/N] ")
        if not res.lower().startswith('y'):
            return None

    with open(keyfile, "r") as f:
        data = f.read()

    key = AesKey.Read(data)

    with open("{0}.encrypted".format(config), "r") as f:
        data = f.read()
    with open(config, "w") as f:
        f.write(zlib.decompress(key.Decrypt(data)))

    return config


class ConfigError(Exception):
    pass



#
# Sesame
#

__title__ = 'sesame'
__version__ = '0.2.0'
__build__ = 0x000020
__author__ = 'Matt Black'
__license__ = 'Simplified BSD License'
__copyright__ = 'Copyright 2013 Matt Black'

try:
    import core
except ImportError:
    pass



import sys

from flask.ext.script import Manager

from sesame.core import encrypt_config, decrypt_config, ConfigError

manager = Manager(usage="Encrypt/decrypt Flask application config")


@manager.option("-c", "--config",
                help="Path to your Flask app config")
@manager.option("-k", "--keyfile",
                help="Path to your keyczar encryption key")
def encrypt(config, keyfile=None):
    "Encrypt a config file"
    try:
        conf_path = encrypt_config(config, keyfile)
        if conf_path is not None:
            print "Application config encrypted at {0}".format(conf_path)

    except ConfigError as e:
        sys.stderr.write("{0}\n".format(e))
        sys.exit(1)


@manager.option("-c", "--config",
                help="Path to your encrypted Flask app config")
@manager.option("-k", "--keyfile",
                help="Path to your keyczar encryption key")
def decrypt(config, keyfile):
    "Decrypt a config file"
    try:
        conf_path = decrypt_config(config, keyfile)
        if conf_path is not None:
            print "Application config decrypted at {0}".format(conf_path)

    except ConfigError as e:
        sys.stderr.write("{0}\n".format(e))
        sys.exit(1)



import argparse
import sys

from sesame.core import encrypt_config, decrypt_config, ConfigError


def entrypoint():
    parser = argparse.ArgumentParser(
        description='Sesame config file encryption and decryption'
    )
    subparsers = parser.add_subparsers(dest='command')

    # parser for the options on encrypt
    pencrypt = subparsers.add_parser('encrypt',
        help='Encrypt a config file'
    )
    pencrypt.set_defaults(func=encrypt_config, term='encrypted')

    pencrypt.add_argument(
        '-c', '--config', action='store', required=True,
        help='Path to your app config')
    pencrypt.add_argument(
        '-k', '--keyfile', action='store',
        help='Path to keyczar encryption key')

    # parser for the options on decrypt
    pdecrypt = subparsers.add_parser('decrypt',
        help='Decrypt a config file'
    )
    pdecrypt.set_defaults(func=decrypt_config, term='decrypted')

    pdecrypt.add_argument(
        '-c', '--config', action='store', required=True,
        help='Path to your encrypted app config')
    pdecrypt.add_argument(
        '-k', '--keyfile', action='store', required=True,
        help='Path to your keyczar encryption key')

    args = parser.parse_args()

    try:
        # call the relevant function
        conf_path = args.func(args.config, args.keyfile)
        if conf_path:
            print 'Application config {0} at {1}'.format(args.term, conf_path)

            if args.keyfile is None:
                print 'Encryption key created at key.pem'

    except ConfigError as e:
        sys.stderr.write('{0}\n'.format(e))
        sys.exit(1)
